from sympy import symbols, atan, cse
from sympy.printing import ccode
import itertools
# Describe variables and parameters
x,y,a,b,gamma,sigma = symbols('sensor.x sensor.y a b gamma sigma')
# Weighted residual
plain_r = (a*x+b-y)/sigma
# Suppressed residual
r = gamma * atan(plain_r / gamma)
# First derivative of r against parameters a and b
drda = r.diff(a)
drdb = r.diff(b)
# Extract common subexpressions to optimize and simplify calculation
(exprs, results) = cse([r,drda, drdb], optimizations='basic')
# Print out the code
for key, value in itertools.chain(exprs, zip(('r','drda','drdb'), results)):
  print('double ' + str(key) + ' = ' + ccode(value, standard='C99') + ';')
# print the result assignment code
print('S += r*r;')
print('G(0) += 2*r*drda;')
print('G(1) += 2*r*drdb;')
print('H(0,0) += 2*drda*drda;')
print('H(0,1) += 2*drda*drdb;')
print('H(1,1) += 2*drdb*drdb;')

